# edit an entry...

use Net::LDAP::Shell::Util qw(debug entry2ldif);
use Net::LDAP::Entry;
use Getopt::Long;

sub main
{

	my ($entry,$string,$tmpfile,$nsum,$osum,$line,%hash,$attr,);
	my ($value,@nvalues,@ovalues,$results);
	my (@natts,@oatts,$dn);

	my ($optresult,@attrs,$usage,$help);

	$usage = "USAGE: edit [--attr <attribute> --attr <attribute>] <LDAP filter>\n";

	$optresult = GetOptions(
		'attr=s'		=> \@attrs,
		'help'		=> \$help,
	);

	if ($help)
	{
		print
"This command is for editing existing objects.  It accepts
a '--attr' argument, as many times as you want, as a specified
list of attributes to edit.  If no attributes are specified,
all are pulled down.

$usage";
		return 0;
	}

	my $rdn = shift @ARGV or warn $usage and return 1;

	debug("rdn is $rdn");

	unless (@attrs)
	{
		@attrs = qw(*);
	}

	$results = $CONFIG{'ldap'}->search(
		'base'	=> $CONFIG{'base'},
		'filter'	=> $rdn,
		'scope'	=> 'one',
		'attrs'	=> \@attrs,
	);
	$results->code and warn $results->error, return 2;

	if ($results->all_entries > 1)
	{
		warn "This command currently not supported on more than one entry.\n";
		warn "Please narrow your search.\n";
		return;
	}

	unless ($results->all_entries)
	{
		warn "Entry $rdn not found\n";
		return 3;
	}

	$entry = $results->pop_entry;

	$tmpfile = "/tmp/edit-$$";

	$string = entry2ldif($entry);

	open TMP, ">$tmpfile" or die "Could not open $tmpfile: $!\n";
	print TMP $string;
	close TMP;

	$osum = qx|sum $tmpfile|;
	chomp $osum;

	$ENV{'EDITOR'} ||= "vi";
	system($ENV{'EDITOR'},$tmpfile);

	$nsum = qx|sum $tmpfile|;
	chomp $nsum;

	if ($osum eq $nsum)
	{
		warn "Entry did not change\n";
		return 0;
	}

	open TMP, $tmpfile or die "Could not open $tmpfile: $!\n";
	while ($line = <TMP>)
	{
		chomp $line;

		# skip blanks and comments, even though I don't know if it's legal
		$line =~ /^\s*#/ and next;
		$line =~ /^\s*$/ and next;

		($attr,$value) = split /: /, $line;

		push @{ $hash{$attr} }, $value;
	}
	close TMP;
	unlink $tmpfile;

	@oatts = $entry->attributes();
	debug("oatts [@oatts]");

	$dn = shift @{ $hash{'dn'} };
	if ($dn ne $entry->dn)
	{
		warn "Object renaming is not supported at this time.\n";
		return 4;
	}
	delete $hash{'dn'};

	foreach $attr (keys %hash)
	{
		debug("working on attr $attr");
		unless (grep /$attr/, @oatts)
		{
			debug("adding $attr");
			$entry->add(
				$attr => $hash{$attr},
			);
			next;
		}
		@ovals = $entry->get_value($attr);
		@nvals = @{ $hash{$attr} };
		debug("compare: $attr ovals [@ovals] nvals [@nvals]");

		if (join('',@ovals) ne join('',@nvals))
		{
			debug("replacing $attr");
			$entry->replace(
				$attr => $hash{$attr},
			);
			next;
		}
	}

	foreach $attr (@oatts)
	{
		unless (grep /$attr/, keys %hash)
		{
			debug("deleting $attr");
			$entry->delete($attr);
		}
	}
	$entry->dump();
	$results = $entry->update($CONFIG{'ldap'});
	$results->code and warn $results->error;
}

1;
