# create a new LDAP object

# you can assume the existence of a %CONFIG variable with everything
# you need for ldap connections
#

use Net::LDAP::Entry qw(new);
use Net::LDAP::Shell qw(shellSearch);
use Net::LDAP::Shell::Util qw(debug error serverType);
use Net::LDAP::Shell::Schema qw(recurse_oc getSchema);
use Getopt::Long;

sub main
{
	my ($usage,$results,@attrs,$optresult,$help,$helptext,@ocs,$debug);

	$usage = "new [--help] <objectclass> <objectclass> ..\n";
	$optresult = GetOptions(
		'help'		=> \$help,
		'debug'		=> \$debug,
	);

	$helptext =
"Create a new LDAP entry.  You can specify which objectclasses
the entry will comprise.
";

	unless ($optresult)
	{
		warn $usage;
		return 1;
	}

	if ($help)
	{
		print $usage,$helptext;
		return;
	}

	@ocs = @ARGV;

	my (%base,@entries,$dse,$type,$schema,%may,%must,%parents);

	%CONFIG = %CONFIG; # shut up, please
	%base = %CONFIG;

	$schema = getSchema($base{'ldap'}) or do
	{
		# we should have an option to just let them create the entry
		# from scratch...
		undef @ocs;
	};

	foreach (@ocs)
	{
		my %oc = recurse_oc($schema,$_) or do
		{
			warn "Could not retrieve objectclass '$_'\n";
			next;
		};

		foreach my $var (keys %oc)
		{
			print "\t$var: ", join(" ", @{ $oc{$var} }), "\n";
		}
		#map {	print "OC: $_ => $oc{$_}\n"; } keys %oc;

		# set all of the info we need
		map {	$parents{$_}++; } @{ $oc{'sup'} }, @{ $oc{'name'} };
		map {	$may{$_}++; } @{ $oc{'may'} };
		map {	$must{$_}++; } @{ $oc{'must'} };
	}

	# okay, by now we know all of the things we need, now we just
	# need to create the tmp file

	my $text = "dn: REPLACE,$base{'base'}\n";

	# put in the parent OCs
	map {	$text .= "objectclass: $_\n"; } sort keys %parents;

	# now set the reqs and options

	# we sort across both at once, both so all atts are sorted but
	# also because given a list of OCs, some may consider an att optional
	# while others consider it mandatory
	foreach (sort (keys %may,keys %must))
	{
		/objectclass/i and next;
		# if anything required it...
		if (exists $must{$_})
		{
			$text .= "$_: \n";
		}
		else # else it's optional
		{
			$text .= "#$_: \n";
		}
	}

	# okay, now we have our empty OC

	my ($tmpfile,$osum,$nsum,%hash,$entry);
	$tmpfile = "/tmp/ldapshnew-$$";

	open TMP, ">$tmpfile" or die "Could not open $tmpfile: $!\n";
	print TMP $text;
	close TMP;

	$osum = qx|sum $tmpfile|;
	chomp $osum;

	$ENV{'EDITOR'} ||= "vi";
	system($ENV{'EDITOR'},$tmpfile);

	$nsum = qx|sum $tmpfile|;
 	chomp $nsum;

	if ($nsum eq $osum)
	{
		warn "File did not change\n";
		return;
	}

 	open TMP, $tmpfile or die "Could not open $tmpfile: $!\n";
	while ($line = <TMP>)
	{
		chomp $line;

		# skip blanks and comments, even though I don't know if it's legal
		$line =~ /^\s*#/ and next;
		$line =~ /^\s*$/ and next;

		# also skip anything that still has the word REPLACE in it
		$line =~ /REPLACE$/ and next;

		# and anything without a value
		$line =~ /: $/ and next;
		($attr,$value) = split /: /, $line;

		push @{ $hash{$attr} }, $value;
	}
	close TMP;
	unlink $tmpfile;

	unless (exists $hash{'dn'})
	{
		die "Failed to retrieve DN; exiting.\n";
	}
	$entry = Net::LDAP::Entry->new();
	$entry->dn(shift @{ $hash{'dn'} });
	delete $hash{'dn'};

	$entry->add(%hash);

	if ($debug)
	{
		$entry->dump;
	}
	else
	{
		$results = $entry->update($base{'ldap'});
		$results->code and die $results->error();
	}
}

1;
